rm(list=ls())

#### Libraries/Packages used
## Loading packages, and installs first if package not present
pckgs <- c("MASS", "latex2exp", "MCMCpack", "leaps", "chron", "foreach", "doParallel")
absent_pckgs <- pckgs[!(pckgs %in% installed.packages()[,"Package"])]
if(length(absent_pckgs)) install.packages(absent_pckgs, dependencies=TRUE)

lapply(pckgs, library, character.only=TRUE)


############################## Initializing ##########################################
set.seed(2015)
source("noisy_data_functions.R")


########################## Reshaping the raw dataset #####################################
#### The raw dataset
dat1 <- read.csv("Data/CombinedABCAll.csv", header=T)
supp_dat <- read.csv("Data/Test_voltage_VS_ampVoltage.csv", header=T)

# The first resistance column
first.col <- which(names(dat1)=="Aligned1")

# Now for just the means for dat1
LA_mat1 <- as.matrix(log(dat1[, first.col:(first.col+4)]))
LA_vec1 <- rowMeans(LA_mat1, na.rm=TRUE)
nA1 <- apply(LA_mat1, 1, function(MM){sum(!(is.na(MM)))})

LNA_mat1 <- as.matrix(log(dat1[, (first.col+5):(first.col+9)]))
LNA_vec1 <- rowMeans(LNA_mat1, na.rm=TRUE)
nNA1 <- apply(LNA_mat1, 1, function(MM){sum(!(is.na(MM)))})


meanData1 <- data.frame(dat1[,1:(first.col-1)], aligned=LA_vec1,
                        non_aligned=LNA_vec1, nA=nA1, nNA=nNA1)
rownames(meanData1) <- NULL
names(meanData1) <- c(names(meanData1)[1:2], 
                      substring(names(meanData1[,3:(first.col-1)]),1,1),
                      names(meanData1[,first.col:ncol(meanData1)]))

highCIndex = (meanData1$C == 0.2)
nonAIndex <- (meanData1$T != 'A')

## Dropping unreliable data (burned samples, boiling occured, etc)
meanData2 <- meanData1[!(highCIndex),]

meanData1b <- meanData2[-c(43, 47, 67), ]
rownames(meanData1b) <- NULL


### The final dataset
meanData <- meanData1b
rownames(meanData) <- NULL

# Adding extra info
meanData$log_ratio <- meanData$non_aligned - meanData$aligned
meanData$E2 <- meanData$E^2
meanData$V2 <- meanData$V^2
meanData$M2 <- meanData$M^2
meanData$I2 <- meanData$I^2
meanData$C2 <- meanData$C^2
meanData$Vo <- 1/meanData$I
meanData$Vo2 <- meanData$Vo^2



################################## Plots #############################################
par(mar = (c(5, 6, 4, 2) + 0.1), cex.lab=1.5, cex.main=1.5, cex.axis=1.5)

plot(c(0,supp_dat$Generator_Voltage), c(0,supp_dat$Electric_Field), 
     xlab=TeX("Input voltage ($Z$)"), ylab=TeX("Electric field ($X$)"),
     main="The generated electric field vs the input voltage", pch=16, col="orange", cex=2)
points(dat1$Voltage, dat1$Electric_Field, pch=4, col="blue", cex=2)
legend("topleft", c("Experimental data","Supplementary Data"), pch=c(4,16), col=c("blue","orange"), cex=1.5)

par(mar = (c(5, 4, 4, 2) + 0.1))

unique_Z <- sort(unique(c(supp_dat$Generator_Voltage, dat1$Voltage)))
X_sds <- sapply(unique_Z, function(zz){
  idx1 <- (dat1$Voltage==zz)
  idx2 <- (supp_dat$Generator_Voltage==zz)
  res <- sd(c(dat1$Electric_Field[idx1], supp_dat$Electric_Field[idx2]))
  
  return(res)
})

plot(unique_Z, X_sds)
xt <- unique_Z[-5] ; yt <- X_sds[-5] ; zt <- yt^2
summary(lm(yt~-1 + xt))
summary(lm(zt~ -1 + xt + I(xt^2)))


zeros.idx1 <- (dat1$voltage != 0)
zeros.idx2 <- (supp_dat$Generator_Voltage != 0)

## The internal noise data
electric_dat <- list(VV=c(dat1$Voltage[zeros.idx1], supp_dat$Generator_Voltage[zeros.idx2]),
                     EE=c(dat1$Electric_Field[zeros.idx1], supp_dat$Electric_Field[zeros.idx2]))


################################## Log-posterior ###########################################
#### First we obtain the design matrix by replicating Remillard et al (2016)

## Setting up the model selection as in Remillard et al (2016)
model_fmla <- as.formula(log_ratio~ T:(E + E2) + (M + M2 + C + C2)*(E + E2))
design_fmla <- as.formula(~ T:(E + E2) + (M + M2 + C + C2)*(E + E2))
null_leaps <- regsubsets(model_fmla, data=meanData, nbest=2, intercept=F)

## Getting the design matrix for the dataset based on the full model
myData <- as.data.frame(model.matrix(design_fmla, data=meanData))
myData$log_ratio <- meanData$log_ratio
names(myData) <- name_rarr(names(myData))

## Using the same model selection settings as in Remillard et al (2016)
myCriterion <- 'cp'
myFullModel_NI <- null_leaps
myRank <- 1
chosen_model_NI <- model_choice(model=myFullModel_NI, type=myCriterion, rank=myRank)

## The model dataframe
best_NI_df <- myData[c('log_ratio', chosen_model_NI$bestTerms)]
rownames(best_NI_df)<- NULL

## The design matrix and response
XX <- as.matrix(best_NI_df[, -1])
YY <- matrix(best_NI_df[, 1], ncol=1)

## The response surface (regression) data
alignment_dat <- list(XX=XX, YY=YY)

## All the data
all_dat <- c(alignment_dat, electric_dat)


#### Now the log-likelihood, log-prior and log-posterior functions
## Log-likelihood function for regression parameters
llike_reg <- function(params, dataset){
  sig <- exp(params[1])
  bet <- matrix(params[-1], ncol=1)
  
  X <- dataset$XX
  Y <- dataset$YY
  EE <- as.numeric(Y - X%*%bet)
  
  ll <- sum(dnorm(EE, mean=0, sd=sig, log=TRUE))
  return(ll)
}

## Log-prior function for regression parameters
lprior_reg <- function(params){
  lpr <- sum(dnorm(params, mean=0, sd=1e5, log=TRUE))
  return(lpr)
}

## Log-likelihood function for hyperparameters on the log-scale
llike_hyper <- function(params, dataset){
  alp <- exp(params[1])
  gam <- exp(params[2])
  
  V <- dataset$VV
  E <- dataset$EE
  
  shap <- alp*V
  rat <- rep(gam, length(shap))
  
  ll <- sum(dgamma(E, shape=shap, rate=rat, log=TRUE))
  return(ll)
}

## Log-prior function for hyperparameters on the log-scale
lprior_hyper <- function(params){
  lpr <- sum(dnorm(params, mean=0, sd=1e5, log=TRUE))
  return(lpr)
}

## Log-posterior for regression parameters
lpost_reg <- function(params, dataset){
  lpst <- llike_reg(params, dataset) + lprior_reg(params)
  
  return(lpst)
}

## Log-posterior for hyper parameters
lpost_hyper <- function(params, dataset){
  lpst <- llike_hyper(params, dataset) + lprior_hyper(params)
  
  return(lpst)
}

## Log-posterior for all parameters
lpost <- function(params, dataset){
  params1 <- params[1:2]
  params2 <- params[-c(1:2)]
  
  lpst <- lpost_reg(params2, dataset) + lpost_hyper(params1, dataset)
  
  return(lpst)
}


## Testing
lpost(runif(3+ncol(XX)), all_dat)

################################ Parameter Esimation (MAPs) #####################
## Parameter initialization
init.par <- runif(3+ncol(XX), -1, 1)
lpost(init.par, all_dat)


### Finding the parameter MAPs
fit <- optim(init.par, lpost, dataset=all_dat, method="BFGS",
             control=list(fnscale=-1, maxit=1e5))

est_alp <- exp(fit$par[1])
est_gam <- exp(fit$par[2])
est_sig <- exp(fit$par[3])
est_bets <- fit$par[-c(1:3)]
names(est_bets) <- chosen_model_NI$bestTerms

est_b0 <- est_bets['C2']
est_b1 <- est_bets['E']
est_b2 <- est_bets['E:TB']
est_b3 <- est_bets['E:TC']
est_b4 <- est_bets['C:E']
est_b5 <- est_bets['E2']
est_b6 <- est_bets['E2:TC']
est_b7 <- est_bets['E2:M']

est_par <- as.numeric(c(est_alp, est_gam, est_sig, est_b0, est_b1, est_b2, est_b3, est_b4,
                        est_b5, est_b6, est_b7))


### An alternative is generating parameter posterior samples instead
fit_mcmc <- MCMCmetrop1R(fun=lpost, theta.init=fit$par, dataset=all_dat,
                         burnin = 2e5, mcmc = 1e6, thin = 100, tune = 0.5, verbose = 5e5,
                         seed=1234, logfun = TRUE, optim.method = "BFGS",
                         optim.control = list(fnscale = -1, trace = 0, REPORT = 1, maxit=50000))
fit_mcmc_df <- as.data.frame(fit_mcmc)
names(fit_mcmc_df) <- c('log(alpha)', 'log(gamma)', 'log(sigma)', paste0('beta_', 0:7))

#plot(as.mcmc(fit_mcmc_df))

orig_mcmc_df <- data.frame(exp(fit_mcmc_df[,1:3]), fit_mcmc_df[,-c(1:3)])
names(orig_mcmc_df) <- c('alpha', 'gamma', 'sigma', paste0('beta_', 0:7))

################################# END ESTIMATION ##################################

################################# CALCULATING Y_STAR ################################
opt_YSTAR <- function(params){
  aa <- params[1]
  cc <- params[2]
  vo <- params[3]
  
  mm <- cc*vo  
  TB <- 0; TC <- 0   ## Sample A only
  
  obj <- est_b0*cc^2 + (est_b1 + est_b2*TB + est_b3*TC + est_b4*cc)*(est_alp/est_gam)*aa + 
    (est_b5 + est_b6*TC + est_b7*mm)*((est_alp/(est_gam^2))*aa + ((est_alp/est_gam)*aa)^2)
  
  return(as.numeric(obj))
}

# Initialization
init_ystar <- c(1.5, 0.05, 0.25)
opt_YSTAR(init_ystar)

# Optimization
fit_ystar <- optim(init_ystar, opt_YSTAR, method="L-BFGS-B", 
                   lower= c(min(meanData$V), min(meanData$C), min(meanData$Vo)),
                   upper=c(max(meanData$V), max(meanData$C), max(meanData$Vo)), 
                   control=list(fnscale=-1))
y_star <- fit_ystar$value
y_star

################################# END CALCULATION ####################################

####################### INITIALIZATION FOR MC-OPTIMIZATION ##################
LB <- min(meanData$V); UB <- max(meanData$V)

## Scale-free noise measure at the posterior mode
SFN2(est_alp, LB, UB)


## Posterior samples of the scale-free noise measure
PS_SFN <- sapply(orig_mcmc_df[, 1], function(aa){
  SFN2(aa, LB, UB)
})

## Posterior mean and 95% PI for scale-free noise measure
mean(PS_SFN); quantile(PS_SFN, probs=c(0.025, 0.975))

############################# END INITIALIZATION ##################################




########################### BEGIN VISUALIZAION AND MC-OPTIMIZATION #######################
############## Start of Monte-Carlo (MC) approx ##################
xh_ref <- XREF(pars=est_par, NN=1e3, lb=LB, ub=UB)


## Visualizing
zd <- seq(LB, UB, length.out=1e2)

# For cc=0.004 and mm=0.001 (vo=0.25)
pdmcs1 <- sapply(zd, function(z){
  PDMC1(pars=est_par, xhRef=xh_ref, c(z, 0.004, 0.25), y_star)
})
# For cc=0.01 and mm=0.001 (vo=0.1)
pdmcs2 <- sapply(zd, function(z){
  PDMC1(pars=est_par, xhRef=xh_ref, c(z, 0.01, 0.1), y_star)
})
# For cc=0.1 and mm=0.025 (vo=0.25)
pdmcs3 <- sapply(zd, function(z){
  PDMC1(pars=est_par, xhRef=xh_ref, c(z, 0.1, 0.25), y_star)
})
# For cc=0.054 and mm=0.009 (vo=0.167)
pdmcs4 <- sapply(zd, function(z){
  PDMC1(pars=est_par, xhRef=xh_ref, c(z, 0.054, 0.167), y_star)
})


## The plots 
par(mar = (c(5, 6, 4, 2) + 0.1), cex.lab=1.5, cex.main=1.5, cex.axis=1.5)

plot(zd, pdmcs1, type="l", xlab="Input voltage, Z", ylab=TeX("$\\eta_2(Z)$"), 
     main="Estimated tail probability \n(M = 0.001 mg and C = 0.004 mg/ml)")
plot(zd, pdmcs2, type="l", xlab="Input voltage, Z", ylab=TeX("$\\eta_2(Z)$"), 
     main="Estimated tail probability \n(M = 0.001 mg and C = 0.01 mg/ml)")
plot(zd, pdmcs3, type="l", xlab="Input voltage, Z", ylab=TeX("$\\eta_2(Z)$"), 
     main="Estimated tail probability \n(M = 0.025 mg and C = 0.1 mg/ml)")
plot(zd, pdmcs4, type="l", xlab="Input voltage, Z", ylab=TeX("$\\eta_2(Z)$"), 
     main="Estimated tail probability \n(M = 0.009 mg and C = 0.054 mg/ml)")

par(mar = (c(5, 4, 4, 2) + 0.1))


#### Optimal point search
## Initialization
init.par <- c(mean(c(LB, UB)), 0.05, 0.15)
PDMC1(pars=est_par, xhRef=xh_ref, init.par, y_star)

## Optim fit
opt_fit <- optim(init.par, PDMC1, pars=est_par, xhRef=xh_ref, y.star=y_star,
                 method="L-BFGS-B", lower= c(LB, min(meanData$C), min(meanData$Vo)),
                 upper=c(UB, max(meanData$C), max(meanData$Vo)), 
                 control=list(fnscale=-1, factr=1e2, REPORT=10))
z_best <- c(opt_fit$par[1], opt_fit$par[2], opt_fit$par[2]*opt_fit$par[3])
z_best

## Electric field samples under full model, based on optimal input voltage
PS_EF <- rgamma(nrow(fit_mcmc), shape=(exp(fit_mcmc[,1])*z_best[1]), rate=exp(fit_mcmc[,2]))
mean(PS_EF)
quantile(PS_EF, probs=c(0.025, 0.975))




############################# Under zero internal noise model #################################
#### First we obtain the design matrix, now replcaing E with V.

## Setting up the model selection as in Remillard et al (2016)
design_fmla_B <- as.formula(~ T:(V + V2) + (M + M2 + C + C2)*(V + V2))

## Getting the design matrix for the dataset based on the full model
myData_B <- as.data.frame(model.matrix(design_fmla_B, data=meanData))
myData_B$log_ratio <- meanData$log_ratio
names(myData_B) <- name_rarr(names(myData_B))

## Replacing all E with V within previous model, and arranging name composition
model_B_terms <- name_rarr(gsub('E','V', chosen_model_NI$bestTerms))

## The model dataframe
model_B_df <- myData_B[c('log_ratio', model_B_terms)]
rownames(model_B_df)<- NULL

## The design matrix and response
XX_B <- as.matrix(model_B_df[, -1])
YY_B <- matrix(model_B_df[, 1], ncol=1)

## The response surface (regression) data
alignment_dat_B <- list(XX=XX_B, YY=YY_B)

## Log-posterior for all parameters
lpost_B <- function(params, dataset){
  lpst <- llike_reg(params, dataset) + lprior_reg(params)
  
  return(lpst)
}


## Testing
lpost_B(runif(1+ncol(XX_B)), alignment_dat_B)

############################## Parameter Esimation (MAPs) #####################
## Parameter initialization
init.par_B <- runif(1+ncol(XX), -1, 1)
lpost_B(init.par_B, alignment_dat_B)


### Finding the parameter MAPs
fit_B <- optim(init.par_B, lpost_B, dataset=alignment_dat_B, method="BFGS",
               control=list(fnscale=-1, maxit=1e4))

est_B_sig <- exp(fit_B$par[1])
est_B_bets <- fit_B$par[-1]
names(est_B_bets) <- model_B_terms

est_B_b0 <- est_B_bets['C2']
est_B_b1 <- est_B_bets['V']
est_B_b2 <- est_B_bets['TB:V']
est_B_b3 <- est_B_bets['TC:V']
est_B_b4 <- est_B_bets['C:V']
est_B_b5 <- est_B_bets['V2']
est_B_b6 <- est_B_bets['TC:V2']
est_B_b7 <- est_B_bets['M:V2']


## Objective function for finding optimal inputs
opt_lm2 <- function(params, y.star){
  aa <- params[1]
  cc <- params[2]
  vo <- params[3]
  
  mm <- cc*vo  
  TB <- 0; TC <- 0   ## Sample A only
  
  mu_est <- est_B_b0*cc^2 + (est_B_b1 + est_B_b2*TB + est_B_b3*TC + est_B_b4*cc)*aa + 
    (est_B_b5 + est_B_b6*TC + est_B_b7*mm)*(aa^2)
  sd_est <- est_B_sig
  
  obj <- 1 - pnorm(y.star, mean=mu_est, sd=sd_est)
  
  return(as.numeric(obj))
}


# Initialization
init.par2 <- c(1.5, 0.05, 0.25)
opt_lm2(init.par2, y.star=y_star)

# Optimization
opt_fit2 <- optim(init.par2, opt_lm2, y.star=y_star, method="L-BFGS-B", 
                  lower= c(LB, min(meanData$C), min(meanData$Vo)),
                  upper=c(UB, max(meanData$C), max(meanData$Vo)), 
                  control=list(fnscale=-1))

# The optimum seting based on no-internal noise model
z_best2 <- c(opt_fit2$par[1], opt_fit2$par[2], opt_fit2$par[2]*opt_fit2$par[3])
z_best2

# Electric field samples under full model, based on optimal input voltage
PS_EF2 <-  rgamma(nrow(fit_mcmc), shape=(exp(fit_mcmc[,1])*z_best2[1]), rate=exp(fit_mcmc[,2]))
mean(PS_EF2)
#quantile(PS_EF2, probs=c(0.025, 0.975))



############################## Simulation for inducing extra internal noise ####################
nsim <- 1e2
s_seq <- seq(0, 1, length.out=nsim+2)[-c(1, (nsim+2))]
alp.sim <- (6/s_seq - 6)/2.5
gam.sim <- alp.sim*(est_gam/est_alp)
dat.size <- nrow(meanData)
meanData_sim <- meanData
electric_sim <- electric_dat
sup.size <- length(electric_dat$EE)


true_par <- est_par

## Number of repetitions to average over
nRep <- 5e2

#### Setting up parallelization
## Calculate the number of logical cores for parallelization
no_cores <- max(detectCores()-1, 1)

## Registering the logical cores
cl<-makeCluster(no_cores)
registerDoParallel(cl)

## Running the parallel simulations
results <- foreach(cc= 1:nsim,
                   .combine=rbind) %dopar% {
                     
                     ## Initialization
                     source('noisy_data_functions.R')
                     set.seed(1234)
                     
                     ## The current alpha and gamma values
                     true_par[1] <- alp.sim[cc]
                     true_par[2] <- gam.sim[cc]
                     
                     aa <- true_par[1]; gg <- true_par[2]
                     
                     ## Storage container for simulation results
                     ans <- matrix(NA, nrow=nRep, ncol=3)
                     for(ii in 1:nRep){
                       
                       ######################### Data generation #################################
                       ## Generating new datasets
                       EE <- rgamma(dat.size, shape=aa*meanData$V, rate=rep(gg, dat.size))
                       meanData_sim$E <- EE
                       meanData_sim$E2 <- EE^2
                       
                       electric_sim$EE <- rgamma(sup.size, shape=aa*electric_sim$VV, 
                                                 rate=rep(gg, sup.size))
                       
                       ## Getting the design matrix for the dataset based on the full model
                       myData_sim <- as.data.frame(model.matrix(design_fmla, data=meanData_sim))
                       names(myData_sim) <- name_rarr(names(myData_sim))
                       
                       ## The model dataframe
                       data_df <- myData_sim[chosen_model_NI$bestTerms]
                       rownames(data_df)<- NULL
                       
                       ## The design matrix and response
                       XX_sim <- as.matrix(data_df)
                       mu_sim <- as.numeric(XX_sim%*%est_bets)
                       YY_sim <- rnorm(dat.size, mean=mu_sim, sd=est_sig)
                       
                       ## The response surface (regression) data
                       alignment_sim <- list(XX=XX_sim, YY=YY_sim)
                       
                       ## saving all the data
                       all_sim <- c(alignment_sim, electric_sim)
                       
                       
                       ############## Parameter Esimation (MAPs) for full model ####################
                       ## Parameter initialization
                       init_reg_par <- runif(1+ncol(XX_sim), -1, 1)
                       init_hyper_par <- runif(2, -1, 1)
                       
                       ## Finding the parameter MAPs
                       fit_reg_sim <- optim(init_reg_par, lpost_reg, dataset=all_sim, 
                                            method="BFGS", control=list(fnscale=-1, maxit=1e5))
                       fit_hyper_sim <- optim(init_hyper_par, lpost_hyper, dataset=all_sim, 
                                              method="BFGS", control=list(fnscale=-1, maxit=1e5))
                       
                       sim_est_alp <- exp(fit_hyper_sim$par[1])
                       sim_est_gam <- exp(fit_hyper_sim$par[2])
                       sim_est_sig <- exp(fit_reg_sim$par[1])
                       sim_est_bets <- fit_reg_sim$par[-1]
                       names(sim_est_bets) <- chosen_model_NI$bestTerms
                       
                       sim_est_b0 <- sim_est_bets['C2']
                       sim_est_b1 <- sim_est_bets['E']
                       sim_est_b2 <- sim_est_bets['E:TB']
                       sim_est_b3 <- sim_est_bets['E:TC']
                       sim_est_b4 <- sim_est_bets['C:E']
                       sim_est_b5 <- sim_est_bets['E2']
                       sim_est_b6 <- sim_est_bets['E2:TC']
                       sim_est_b7 <- sim_est_bets['E2:M']
                       
                       sim_est_par <- as.numeric(c(sim_est_alp, sim_est_gam, sim_est_sig, sim_est_b0, 
                                                   sim_est_b1, sim_est_b2, sim_est_b3, sim_est_b4, 
                                                   sim_est_b5, sim_est_b6, sim_est_b7))
                       
                       ######################### Y_STAR calculation ##############################
                       opt_YSTAR_sim <- function(params, model_pars){
                         ee <- params[1]
                         cc <- params[2]
                         vo <- params[3]
                         
                         mm <- cc*vo  
                         TB <- 0; TC <- 0   ## Sample A only
                         
                         obj <- model_pars[4]*cc^2 + 
                           (model_pars[5] + model_pars[6]*TB + 
                              model_pars[7]*TC + model_pars[8]*cc)*(model_pars[1]/model_pars[2])*ee + 
                           (model_pars[9] + model_pars[10]*TC + 
                              model_pars[11]*mm)*((model_pars[1]/(model_pars[2]^2))*ee +                                                      ((model_pars[1]/model_pars[2])*ee)^2)
                         
                         return(as.numeric(obj))
                       }
                       
                       # Initialization
                       init_ystar <- c(1.5, 0.05, 0.25)
                       
                       # Optimization
                       fit_ystar_sim_est <- optim(init_ystar, opt_YSTAR_sim, 
                                                  model_pars=sim_est_par, method="L-BFGS-B", 
                                              lower= c(min(meanData$V), min(meanData$C), 
                                                       min(meanData$Vo)),
                                              upper=c(max(meanData$V), max(meanData$C), 
                                                      max(meanData$Vo)), 
                                              control=list(fnscale=-1, maxit=1e5))
                      
                       y_star_est <- fit_ystar_sim_est$value
                       y_star_true <- y_star
                       
                       
                       ################# Optimal point search for full model #######################
                       ## Initialization
                       init.par <- c(mean(c(LB, UB)), 0.05, 0.15)
                       xh_ref_sim <- XREF(pars=sim_est_par, NN=1e3, lb=LB, ub=UB)
                       
                       ## Optim fit
                       opt_fit_sim <- optim(init.par, PDMC1, pars=sim_est_par, 
                                            xhRef=xh_ref_sim, y.star=y_star_est,
                                            method="L-BFGS-B", 
                                            lower= c(LB, min(meanData$C), min(meanData$Vo)),
                                            upper=c(UB, max(meanData$C), max(meanData$Vo)), 
                                            control=list(fnscale=-1, maxit=1e5))
                       z_best_sim <- c(opt_fit_sim$par[1], opt_fit_sim$par[2], 
                                       opt_fit_sim$par[2]*opt_fit_sim$par[3])
                       
                       
                       
                       ################## Under zero internal noise model ##########################
                       #### First we obtain the design matrix, now replcaing E with V.
                       ## Getting the design matrix for the dataset based on the full model
                       myData_B_sim <- as.data.frame(model.matrix(design_fmla_B, data=meanData_sim))
                       names(myData_B_sim) <- name_rarr(names(myData_B_sim))
                       
                       ## The model dataframe
                       model_B_sim <- myData_B_sim[model_B_terms]
                       rownames(model_B_sim)<- NULL
                       
                       ## The design matrix and response
                       XX_B_sim <- as.matrix(model_B_sim)
                       
                       ## The response surface (regression) data
                       alignment_sim_B <- list(XX=XX_B_sim, YY=YY_sim)
                       
                       ############################## Parameter Esimation (MAPs) #####################
                       ## Parameter initialization
                       init.par_B <- runif(1+ncol(XX_sim), -1, 1)
                       
                       ### Finding the parameter MAPs
                       fit_B_sim <- optim(init.par_B, lpost_B, dataset=alignment_sim_B, 
                                          method="BFGS",
                                          control=list(fnscale=-1, maxit=1e5))
                       
                       sim_est_B_sig <- exp(fit_B_sim$par[1])
                       sim_est_B_bets <- fit_B_sim$par[-1]
                       names(sim_est_B_bets) <- model_B_terms
                       
                       sim_est_B_b0 <- sim_est_B_bets['C2']
                       sim_est_B_b1 <- sim_est_B_bets['V']
                       sim_est_B_b2 <- sim_est_B_bets['TB:V']
                       sim_est_B_b3 <- sim_est_B_bets['TC:V']
                       sim_est_B_b4 <- sim_est_B_bets['C:V']
                       sim_est_B_b5 <- sim_est_B_bets['V2']
                       sim_est_B_b6 <- sim_est_B_bets['TC:V2']
                       sim_est_B_b7 <- sim_est_B_bets['M:V2']
                       
                       
                       ## Objective function for finding optimal inputs
                       opt_lm2_sim <- function(params, y.star){
                         aa <- params[1]
                         cc <- params[2]
                         vo <- params[3]
                         
                         mm <- cc*vo  
                         TB <- 0; TC <- 0   ## Sample A only
                         
                         mu_est <- sim_est_B_b0*cc^2 + 
                           (sim_est_B_b1 + sim_est_B_b2*TB + sim_est_B_b3*TC + sim_est_B_b4*cc)*aa + 
                           (sim_est_B_b5 + sim_est_B_b6*TC + sim_est_B_b7*mm)*(aa^2)
                         sd_est <- sim_est_B_sig
                         
                         obj <- 1 - pnorm(y.star, mean=mu_est, sd=sd_est)
                         
                         return(as.numeric(obj))
                       }
                       
                       
                       # Initialization
                       init.par2 <- c(1.5, 0.05, 0.25)
                       
                       # Optimization
                       opt_fit2_sim <- optim(init.par2, opt_lm2_sim, y.star=y_star_est, 
                                             method="L-BFGS-B", 
                                             lower= c(LB, min(meanData$C), min(meanData$Vo)),
                                             upper=c(UB, max(meanData$C), max(meanData$Vo)), 
                                             control=list(fnscale=-1))
                       
                       # The optimum seting based on no-internal noise model
                       z_best2_sim <- c(opt_fit2_sim$par[1], opt_fit2_sim$par[2], 
                                        opt_fit2_sim$par[2]*opt_fit2_sim$par[3])
                       
                       YY <- Gen_post(list(z_best_sim, z_best2_sim), par=true_par, num_sim=1e5)
                       
                       ## The tail behaviour for MC generated data
                       cump1 <- YY[[1]] > y_star_true
                       cump2 <- YY[[2]] > y_star_true
                       
                       ## The tail probabilities for MC generated data
                       pdms <- c(mean(cump1), mean(cump2))
                       
                       ## The probaility of domination for model 1 over model 2, 
                       ## using the MC generated data
                       p1p2 <- mean(YY[[1]]>YY[[2]])
                       
                       ans[ii,] <- c(pdms, p1p2)
                     }
                     
                     ## Simulation results for one value of unit-free noise measure
                     res <- colMeans(ans)
                     
                     return(res)
                   }

stopCluster(cl)

sim_res <- as.data.frame(cbind(s_seq, results))
names(sim_res) <- c('SFN', 'PDM1', 'PDM2', 'PDOM')


## Plot simulation results
par(mar = (c(5, 6, 4, 2) + 0.1), cex.lab=1.5, cex.main=1.5, cex.axis=1.5)
plot(sim_res$SFN, sim_res$PDM1, type="l", ylim=c(0, 0.6),
     xlab=TeX("$S(\\alpha, \\gamma)$"),
     ylab=TeX("$J(Z(\\alpha, \\gamma))$"),
     main="Approximate tail probabilities", col='red', lwd=2)
lines(sim_res$SFN, sim_res$PDM2, lty=2, col='blue', lwd=2)
legend("topright", c(TeX("$J(\\tilde{Z}_{Full})$"),
                     TeX("$J(\\tilde{Z}_{Mis})$")),
       lty=c(1,2), col=c("red","blue"), lwd=2, cex=1.5)

plot(sim_res$SFN, sim_res$PDOM, xlab=TeX("$S(\\alpha, \\gamma)$"),
     ylab=TeX("$K(\\alpha, \\gamma)$"), main="Approximate probability of domination",
     type="l", col="blue", lwd=2, cex=1.5)
